/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum;

import java.util.UUID;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.ProviderManager;
import org.springframework.security.authentication.dao.DaoAuthenticationProvider;
import org.springframework.security.config.Customizer;
import org.springframework.security.config.annotation.authentication.configuration.AuthenticationConfiguration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.factory.PasswordEncoderFactories;
import org.springframework.security.saml2.provider.service.registration.RelyingPartyRegistrationRepository;
import org.springframework.security.saml2.provider.service.web.DefaultRelyingPartyRegistrationResolver;
import org.springframework.security.saml2.provider.service.web.RelyingPartyRegistrationResolver;
import org.springframework.security.saml2.provider.service.web.authentication.OpenSaml4AuthenticationRequestResolver;
import org.springframework.security.saml2.provider.service.web.authentication.Saml2AuthenticationRequestResolver;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import de.ozgcloud.antragsraum.mocks.MockLogoutSuccessHandler;
import de.ozgcloud.antragsraum.mocks.MockSuccessHandler;
import de.ozgcloud.antragsraum.security.AntragsraumLogoutSuccessHandler;
import de.ozgcloud.antragsraum.security.AntragsraumProperties;
import de.ozgcloud.antragsraum.security.BayernIdProperties;
import de.ozgcloud.antragsraum.security.BayernIdSaml2Extension;
import de.ozgcloud.antragsraum.security.InMemoryUserDetailService;
import de.ozgcloud.antragsraum.security.JwtTokenFilter;
import de.ozgcloud.antragsraum.security.SamlUrlAuthenticationSuccessHandler;
import de.ozgcloud.antragsraum.security.SecurityProvider;
import de.ozgcloud.antragsraum.security.User;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.server.security.authentication.BasicGrpcAuthenticationReader;
import net.devh.boot.grpc.server.security.authentication.GrpcAuthenticationReader;

@Configuration
@EnableWebSecurity
@RequiredArgsConstructor
public class SecurityConfiguration {
	public static final String OPTIONS = "OPTIONS";
	public static final String GET = "GET";
	@NonNull
	private final BayernIdSaml2Extension bayernIdSaml2Extension;
	@NonNull
	private final BayernIdProperties bayernIdProperties;
	@NonNull
	private final UserDetailsService userDetailsService;
	@NonNull
	private final JwtTokenFilter tokenAuthenticationFilter;
	@NonNull
	private final AntragsraumProperties properties;
	@Value("${ozgcloud.mock.auth:false}")
	private boolean mockAuth;
	@Value("${ozgcloud.mock.password:}")
	private String password;

	@Bean
	public SecurityProvider securityProvider() {
		return new SecurityProvider();
	}

	@Bean
	Saml2AuthenticationRequestResolver authenticationRequestResolver(RelyingPartyRegistrationRepository registrations) {
		RelyingPartyRegistrationResolver registrationResolver =
		  new DefaultRelyingPartyRegistrationResolver(registrations);
		OpenSaml4AuthenticationRequestResolver authenticationRequestResolver =
		  new OpenSaml4AuthenticationRequestResolver(registrationResolver);
		authenticationRequestResolver.setAuthnRequestCustomizer(context -> {
			context.getAuthnRequest().setForceAuthn(true);
			context.getAuthnRequest().setExtensions(bayernIdSaml2Extension.createAkdbExtension());
		});
		return authenticationRequestResolver;
	}

	@Bean
	public AuthenticationManager authenticationManager(AuthenticationConfiguration authenticationConfiguration)
	  throws Exception {
		if (mockAuth) {
			DaoAuthenticationProvider authenticationProvider = new DaoAuthenticationProvider();
			authenticationProvider.setUserDetailsService(userDetailsService);
			authenticationProvider.setPasswordEncoder(PasswordEncoderFactories.createDelegatingPasswordEncoder());

			ProviderManager providerManager = new ProviderManager(authenticationProvider);
			providerManager.setEraseCredentialsAfterAuthentication(false);

			return providerManager;
		}

		return authenticationConfiguration.getAuthenticationManager();
	}

	@Bean
	public SecurityFilterChain filterChain(HttpSecurity http) throws Exception {
		http
		  .authorizeHttpRequests(authorize -> authorize
			.requestMatchers("/", "/actuator/**", "/swagger-ui/**", "/v3/api-docs/**", "/error", "/favicon.ico",
			  "/auth/**", "/login", "/api/e2e/**").permitAll()
			.requestMatchers("/api/**").authenticated()
			.anyRequest().denyAll())
		  .addFilterBefore(tokenAuthenticationFilter, UsernamePasswordAuthenticationFilter.class)
		  .csrf(AbstractHttpConfigurer::disable);

		if (mockAuth) {
			var pwEnc = PasswordEncoderFactories.createDelegatingPasswordEncoder();
			var user = User.builder().id(UUID.randomUUID().toString()).username("test").password(pwEnc.encode(password)).build();
			((InMemoryUserDetailService) userDetailsService).addUser(UUID.randomUUID().toString(), user);
			http.httpBasic(Customizer.withDefaults())
			  .formLogin(formLoginConfigurer -> formLoginConfigurer.successHandler(
				new MockSuccessHandler(userDetailsService)))
			  .logout(logoutConfigurer -> logoutConfigurer.logoutSuccessHandler(new MockLogoutSuccessHandler()));
		} else {
			http.saml2Login(
				saml2LoginConfigurer -> saml2LoginConfigurer.successHandler(getSamlUrlAuthenticationSuccessHandler()))
			  .saml2Logout(Customizer.withDefaults())
			  .logout(logoutConfigurer -> logoutConfigurer.logoutSuccessHandler(logoutSuccessHandler()));
		}

		return http.build();
	}

	AuthenticationSuccessHandler getSamlUrlAuthenticationSuccessHandler() {
		return new SamlUrlAuthenticationSuccessHandler(bayernIdProperties.getRedirectUrl(), userDetailsService);
	}

	AntragsraumLogoutSuccessHandler logoutSuccessHandler() {
		var handler = new AntragsraumLogoutSuccessHandler(userDetailsService);
		handler.setDefaultTargetUrl(properties.getLogoutSuccessUrl());
		return handler;
	}

	@Bean
	public WebMvcConfigurer corsConfigurer() {
		return new WebMvcConfigurer() {
			@Override
			public void addCorsMappings(@NonNull CorsRegistry registry) {
				registry.addMapping("/auth/**").allowedOrigins(properties.getAuthOrigins())
				  .allowedMethods(GET, OPTIONS, "POST").allowCredentials(true);
				registry.addMapping("/api/**").allowedOrigins(properties.getApiOrigins())
				  .allowedMethods(GET, OPTIONS, "PUT", "POST").allowCredentials(true);
				registry.addMapping("/**").allowedOrigins(properties.getOtherOrigins()).allowedMethods(GET, OPTIONS)
				  .allowCredentials(true);
			}
		};
	}

	@Bean
	public GrpcAuthenticationReader grpcAuthenticationReader() {
		return new BasicGrpcAuthenticationReader();
	}
}
