/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.concurrent.CompletableFuture;

import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.antragsraum.common.VirusFoundException;
import de.ozgcloud.apilib.file.OzgCloudUploadFile;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
@RequiredArgsConstructor
class FileService {
	private final @NonNull FileRemoteService fileRemoteService;
	private final @NonNull VirusScannerClient scanner;

	OzgFile getFile(String fileId, String address) {
		return fileRemoteService.getFile(fileId, address);
	}

	void downloadFileContent(String fileId, OutputStream out, String address) {
		fileRemoteService.downloadFileContent(fileId, out, address);
	}

	CompletableFuture<String> upload(String vorgangId, String address, MultipartFile file) {
		return fileRemoteService.uploadFile(createUploadFile(vorgangId, file), address);
	}

	OzgUploadFile createUploadFile(String vorgangId, MultipartFile file) throws VirusFoundException {
		if (isClean(file)) {
			LOG.info("File {} is clean, preparing upload", file.getOriginalFilename());
			return OzgUploadFile.builder()
			  .fileData(OzgCloudUploadFile.builder()
				.fileName(file.getOriginalFilename())
				.contentType(file.getContentType())
				.vorgangId(vorgangId)
				.build())
			  .fileContent(getInputStream(file))
			  .build();
		}

		LOG.error("Found viruses in file {}", file.getOriginalFilename());
		throw new VirusFoundException(vorgangId);
	}

	boolean isClean(MultipartFile file) {
		return scanner.scan(file).isEmpty();
	}

	private InputStream getInputStream(MultipartFile file) {
		try {
			return file.getInputStream();
		} catch (IOException e) {
			LOG.error("Error reading file data as stream.", e);
			throw new TechnicalException(e);
		}
	}
}
