/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import java.util.List;
import java.util.Locale;
import java.util.Optional;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.util.unit.DataSize;
import org.springframework.web.multipart.MultipartFile;

@RequiredArgsConstructor
@Component
class FileValidator {
    public static final String DEFAULT_FILE_SUFFIXES_STRING = "pdf,doc,docx,xls,xlsx,txt,png,jpg,jpeg,tif,tiff";
    static final long DEFAULT_MAX_SIZE = 100;

    private final FileProperties fileProperties;

    public boolean isValid(MultipartFile value) {
        var allowedSuffixes = getAllowedFileTypeSuffixes(fileProperties);

        var fileNameOptional = Optional.ofNullable(value.getOriginalFilename());

        boolean isValid =
            fileNameOptional.map(fileName -> allowedSuffixes.contains(getFileNameSuffix(fileName).toLowerCase()))
                .orElse(Boolean.FALSE);

        return isValid && fileSizeIsValid(value.getSize());
    }

    List<String> getAllowedFileTypeSuffixes(FileProperties fileProperties) {
        var suffixedString = Optional.ofNullable(fileProperties.getAllowedSuffixes());
        return suffixedString.map(val -> List.of(val.split(",")))
            .orElse(List.of(DEFAULT_FILE_SUFFIXES_STRING.split(",")));
    }

    String getFileNameSuffix(@NonNull String fileName) {
        return fileName.toLowerCase(Locale.ROOT).substring(fileName.lastIndexOf(".") + 1);
    }

    public boolean fileSizeIsValid(long fileSize) {
        var maxFileSize = Optional.ofNullable(fileProperties.getMaxFileSize());

        return maxFileSize.flatMap(
                dataSize -> {
                    var val = dataSize.toBytes() > DataSize.ofBytes(fileSize).toBytes();
                    return Optional.of(val);
                })
            .orElse(DataSize.ofMegabytes(DEFAULT_MAX_SIZE).toBytes() > DataSize.ofBytes(fileSize).toBytes());
    }
}
