/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.callcontext;

import de.ozgcloud.antragsraum.common.AuthenticationHelper;
import de.ozgcloud.antragsraum.security.User;
import de.ozgcloud.vorgang.grpc.command.GrpcCallContext;
import io.grpc.Metadata;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.stereotype.Service;

import java.nio.charset.Charset;
import java.util.Objects;

@Log4j2
@Service
@RequiredArgsConstructor
public class ContextService {
    public static final String CLIENT_NAME = "Antragsraum";
    public static final String KEY_CLIENT_NAME = "CLIENT_NAME-bin";
    public static final String KEY_REQUEST_ID = "REQUEST_ID-bin";
    public static final String KEY_JWT_TOKEN = "JWT_TOKEN-bin";
    public static final String NO_SAML_TOKEN = "--not set--";

    private final @NonNull AuthenticationHelper authenticationHelper;
    RequestAttributes requestAttributes;

    public GrpcCallContext createCallContext() {
        return GrpcCallContext.newBuilder()
                .setClient(CLIENT_NAME)
                .setRequestId(getRequestId())
                .build();
    }

    public Metadata buildCallContextMetadata() {
        var metadata = new Metadata();

        metadata.put(createKeyOf(KEY_CLIENT_NAME), getClientName().getBytes(Charset.defaultCharset()));
        metadata.put(createKeyOf(KEY_REQUEST_ID), getRequestId().getBytes(Charset.defaultCharset()));
        metadata.put(createKeyOf(KEY_JWT_TOKEN), getJwtToken().getBytes(Charset.defaultCharset()));

        return metadata;
    }

    public static Metadata.Key<byte[]> createKeyOf(String key) {
        return Metadata.Key.of(key, Metadata.BINARY_BYTE_MARSHALLER);
    }

    String getClientName() {
        return CLIENT_NAME;
    }

    public String getSamlResponse() {
        String tokenValue = NO_SAML_TOKEN;
        var auth = authenticationHelper.getAuthentication();

        if (auth instanceof UsernamePasswordAuthenticationToken userToken && userToken.getPrincipal() instanceof User user) {
            tokenValue = user.getSamlToken();
        }

        return tokenValue;
    }

    String getRequestId() {
        return Objects.isNull(requestAttributes) ? "- no request scope -" : requestAttributes.getRequestId();
    }

    String getJwtToken() {
        var auth = authenticationHelper.getAuthentication();
        if (auth instanceof UsernamePasswordAuthenticationToken usernamePasswordAuthenticationToken) {
            return (String) usernamePasswordAuthenticationToken.getCredentials();
        }

        return "";
    }
}
