/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.mocks;

import de.ozgcloud.antragsraum.common.NotFoundException;
import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.nachrichten.antragraum.GrpcFindRueckfragenResponse;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrage;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageAnswer;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcBinaryFile;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileMetaData;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicReference;

import static de.ozgcloud.antragsraum.mocks.GrpcDataCreator.*;

@Log4j2
@Component
@RequiredArgsConstructor
class StubDataRepository {
	private final Map<String, List<String>> messageStorage = new ConcurrentHashMap<>();
	private final Map<String, GrpcRueckfrage> nachrichtStorage = new ConcurrentHashMap<>();
	private final Map<String, GrpcUploadBinaryFileMetaData> fileStorage = new ConcurrentHashMap<>();
	private final Map<String, byte[]> fileContentStorage = new ConcurrentHashMap<>();

	public GrpcFindRueckfragenResponse getRueckfragenResponse(String postfachId) {
		reset();
		if (messageStorage.get(postfachId) != null) {
			var nachrichten = messageStorage.get(postfachId).stream().map(nachrichtStorage::get).toList();
			return GrpcFindRueckfragenResponse.newBuilder().clearRueckfrage().addAllRueckfrage(nachrichten).build();
		} else {
			throw new NotFoundException(GrpcFindRueckfragenResponse.class, postfachId);
		}
	}

	void addFileContent(String fileId, byte[] content) {
		fileContentStorage.put(fileId, content);
	}

	String addFile(GrpcUploadBinaryFileMetaData file) {
		var fileId = RandomStringUtils.randomAlphanumeric(16);
		fileStorage.put(fileId, file);
		return fileId;
	}

	boolean addReply(GrpcRueckfrageAnswer answer) {
		boolean success = true;

		var reply = nachrichtStorage.get(answer.getRueckfrageId());
		if (reply != null) {
			var attachmentsIds = answer.getAttachmentFileIdList().stream()
					.map(this::getUploadedAnswerAttachment)
					.filter(Optional::isPresent)
					.map(Optional::get)
					.map(GrpcBinaryFile::getId)
					.toList();
			var filledAnswer = answer.toBuilder().addAllAttachmentFileId(attachmentsIds).build();

			var replyBuilder = reply.toBuilder()
					.addAnswers(filledAnswer)
					.setAnsweredAt(LocalDateTime.now().toString());

			nachrichtStorage.put(answer.getRueckfrageId(), replyBuilder.build());
		} else {
			throw new NotFoundException(GrpcRueckfrageAnswer.class, answer.getRueckfrageId());
		}

		return success;
	}

	Optional<GrpcBinaryFile> getUploadedAnswerAttachment(String id) {
		var fileDataOpt = getFileMetaData(id);

		AtomicReference<Optional<GrpcBinaryFile>> res = new AtomicReference<>(Optional.empty());

		fileDataOpt.ifPresent(fileMetaData -> res.set(Optional.of(GrpcBinaryFile.newBuilder()
				.setContentType(fileMetaData.getContentType())
				.setName(fileMetaData.getFileName())
				.setSize(fileMetaData.getSize())
				.setId(id).build()))
		);

		return res.get();
	}

	Optional<GrpcUploadBinaryFileMetaData> getFileMetaData(String fileId) {
		if (FILE_ID_1.equals(fileId)) {
			return Optional.of(GrpcUploadBinaryFileMetaData.newBuilder()
					.setFileName(FILE_NAME)
					.setVorgangId(VORGANG_ID_1)
					.setSize(FILE_CONTENT.length)
					.setContentType(MediaType.TEXT_PLAIN_VALUE)
					.build());
		}
		return Optional.ofNullable(fileStorage.get(fileId));
	}

	Optional<byte[]> getFileContent(String fileId) {
		return Optional.ofNullable(fileContentStorage.get(fileId));
	}


	public void reset() {
		messageStorage.clear();
		fileStorage.clear();

		nachrichtStorage.put(NACHRICHT_ID_1_0, getRueckfrageBuilder(VORGANG_ID_1).build());
		messageStorage.put(POSTFACH_ID_1, List.of(NACHRICHT_ID_1_0));

		nachrichtStorage.put(NACHRICHT_ID_2_0, getRueckfrageBuilder(VORGANG_ID_2).build());
		nachrichtStorage.put(NACHRICHT_ID_2_1, getRueckfrageBuilder(VORGANG_ID_2).build());
		messageStorage.put(POSTFACH_ID_2, List.of(NACHRICHT_ID_2_0, NACHRICHT_ID_2_1));
	}

	public void setFile(GrpcBinaryFile file, String vorgangId) {
		fileStorage.put(file.getId(), GrpcUploadBinaryFileMetaData.newBuilder()
				.setFileName(file.getName())
				.setVorgangId(vorgangId)
				.setSize(file.getSize())
				.setContentType(file.getContentType())
				.build());
	}

	public String addReply(GrpcRueckfrage rueckfrage, NachrichtEvent event) {
		String key = event.postfachId() + "-" + event.address();
		nachrichtStorage.put(key, rueckfrage);

		var nachrichtenListOpt = Optional.ofNullable(messageStorage.get(event.postfachId()));
		nachrichtenListOpt.ifPresentOrElse(nachrichtenList -> {
			if (!nachrichtenList.contains(key)) {
				var nachrichtenIds = new ArrayList<>(nachrichtenList);
				nachrichtenIds.add(rueckfrage.getId());
				messageStorage.put(event.postfachId(), nachrichtenIds);
			}
		}, () -> messageStorage.put(event.postfachId(), List.of(rueckfrage.getId())));
		return rueckfrage.getId();
	}

}
