/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.nachricht;

import java.time.DateTimeException;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.List;
import java.util.Objects;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import de.ozgcloud.antragsraum.attachments.FileRemoteService;
import de.ozgcloud.antragsraum.attachments.OzgFile;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrage;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageAnswer;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
@RequiredArgsConstructor
class NachrichtMapper {
	private final @NonNull FileRemoteService fileRemoteService;

	Nachricht fromGrpcRueckfrage(GrpcRueckfrage grpcRueckfrage, String address) {
		var builder = Nachricht.builder()
		  .vorgangId(grpcRueckfrage.getVorgangId())
		  .id(grpcRueckfrage.getId())
		  .address(address)
		  .topicTitle(grpcRueckfrage.getVorgangName())
		  .message(grpcRueckfrage.getText())
		  .replyNachrichten(fromGrpcRueckfrageAnswer(grpcRueckfrage, address))
		  .date(getUnixTimestamp(grpcRueckfrage.getSentAt()))
		  .attachments(getAttachments(grpcRueckfrage.getAttachmentFileIdList(), address))
		  .trustLevel(NachrichtTrustLevel.LOW);

		return builder.build();
	}

	private List<OzgFile> getAttachments(List<String> fileIds, String address) {
		return fileIds.stream().filter(StringUtils::isNotEmpty).map(fileId -> fileRemoteService.getFile(fileId, address)).filter(Objects::nonNull)
		  .toList();
	}

	private List<ReplyNachricht> fromGrpcRueckfrageAnswer(GrpcRueckfrage rueckfrage, String address) {
		return rueckfrage.getAnswersList().stream()
		  .map(answer -> {
			  var builder = ReplyNachricht.builder()
				.id(rueckfrage.getId())
				.address(address)
				.message(answer.getAnswerText())
				.date(getUnixTimestamp(rueckfrage.getAnsweredAt()))
				.attachments(getAttachments(answer.getAttachmentFileIdList(), address));
			  return builder.build();
		  }).toList();
	}

	private long getUnixTimestamp(String dateString) {
		long timestamp = 0;
		try {
			timestamp = LocalDateTime.parse(dateString).toEpochSecond(ZoneOffset.UTC);
		} catch (DateTimeException e) {
			LOG.warn("Invalid send date [{}] received for Nachricht. Setting it to 0 (=1970-01-01)", dateString);
		}
		return timestamp;
	}

	GrpcRueckfrageAnswer toGrpcRueckfrageAnswer(ReplyNachricht nachricht) {
		var builder = GrpcRueckfrageAnswer.newBuilder()
		  .setRueckfrageId(nachricht.id())
		  .setAnswerText(nachricht.message());

		if (Objects.nonNull(nachricht.attachments())) {
			builder.addAllAttachmentFileId(nachricht.attachments().stream()
			  .map(OzgFile::id).toList());
		}
		return builder.build();
	}
}
