/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.nachricht;

import java.util.List;
import java.util.Objects;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import de.ozgcloud.antragsraum.callcontext.ContextService;
import de.ozgcloud.antragsraum.common.SendTimeoutException;
import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.nachrichten.antragraum.GrpcFindRueckfragenRequest;
import de.ozgcloud.nachrichten.antragraum.GrpcSendRueckfrageAnswerRequest;
import de.ozgcloud.nachrichten.antragraum.GrpcSendRueckfrageAnswerResponse;
import io.grpc.StatusRuntimeException;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
@RequiredArgsConstructor
class NachrichtenRemoteService {
	public static final String STATUS_FINISHED = "FINISHED";

	private final @NonNull NachrichtenGrpcClient grpcClient;
	private final @NonNull NachrichtProperties properties;
	private final @NonNull ContextService contextService;
	private final @NonNull NachrichtMapper nachrichtMapper;

	List<Nachricht> findRueckfragen(String postfachId, String address) {
		try {
			var rfBuilder = GrpcFindRueckfragenRequest.newBuilder()
			  .setSamlToken(contextService.getSamlResponse())
			  .setPostfachId(postfachId);
			if (Objects.nonNull(contextService.getSamlResponse())) {
				rfBuilder.setSamlToken(contextService.getSamlResponse());
			}
			var reply = grpcClient.getFindRueckfragen(rfBuilder.build(), address);

			return reply.getRueckfrageList().stream()
			  .map(rueckfrage -> nachrichtMapper.fromGrpcRueckfrage(rueckfrage, address)).toList();
		} catch (StatusRuntimeException e) {
			LOG.error("Error loading nachricht of postfach " + postfachId, e);
		}

		return List.of();
	}

	boolean sendAnswer(ReplyNachricht nachricht, String address) {
		GrpcSendRueckfrageAnswerResponse response = grpcClient.answerRueckfrage(
		  GrpcSendRueckfrageAnswerRequest.newBuilder()
			.setAnswer(nachrichtMapper.toGrpcRueckfrageAnswer(nachricht))
			.setSamlToken(contextService.getSamlResponse())
			.build(), address);

		return waitUntilCommandIsFinished(address, response.getCommandId());
	}

	boolean waitUntilCommandIsFinished(String address, String commandId) {
		var auth = SecurityContextHolder.getContext().getAuthentication();
		ExecutorService executor = Executors.newSingleThreadScheduledExecutor();
		Future<Boolean> future = executor.submit(() -> checkCommandIsFinished(address, commandId, auth));
		try {
			return getSendResult(future);
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			throw new TechnicalException(e);
		} catch (ExecutionException e) {
			throw new TechnicalException(e);
		} catch (TimeoutException e) {
			throw new SendTimeoutException(commandId, e);
		} finally {
			try {
				executor.shutdown();
			} catch (Exception e) {
				LOG.error("Error shutting down executor. Closing it", e);
				executor.close();
			}
		}
	}

	Boolean checkCommandIsFinished(String address, String commandId, Authentication auth) throws InterruptedException {
		SecurityContextHolder.getContext().setAuthentication(auth);

		boolean notFinished = true;
		while (notFinished) {
			var resCommand = grpcClient.getCommand(address, commandId);
			notFinished = !STATUS_FINISHED.equals(resCommand.getStatus());

			TimeUnit.MILLISECONDS.sleep(properties.getSendPollIntervalDuration().toMillis());
		}

		return true;
	}

	Boolean getSendResult(Future<Boolean> future) throws InterruptedException, ExecutionException, TimeoutException {
		return future.get(properties.getTimeoutDuration().toMillis(), TimeUnit.MILLISECONDS);
	}
}
