/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.nachricht;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Collection;
import java.util.List;

import org.springframework.stereotype.Service;

import de.ozgcloud.antragsraum.common.InsufficientTrustLevelException;
import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.antragsraum.events.NachrichtEventService;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
@RequiredArgsConstructor
class NachrichtenService {
	static final String TOPIC_LINK_RELATIONSHIP_NAME = "topic";

	private final @NonNull NachrichtEventService nachrichtEventService;
	private final @NonNull NachrichtenRemoteService nachrichtenRemoteService;
	private final @NonNull NachrichtenTrustLevelService nachrichtenTrustLevelService;

	List<TopicHeader> getTopicHeadersOfPostfach(String postfachId) {
		var topicHeaders = TopicHeaderMapper.fromNachrichten(getRueckfragenOfPostfach(postfachId));

		return linkTopicHeaders(topicHeaders, postfachId);
	}

	private List<TopicHeader> linkTopicHeaders(List<TopicHeader> topicHeaders, String postfachId) {
		return topicHeaders.stream().peek(topicHeader -> {
			topicHeader.add(linkTo(NachrichtenController.class).withSelfRel());

			if (nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(topicHeader.getTrustLevel())) {
				topicHeader.add(
				  linkTo(methodOn(NachrichtenController.class).getTopicOfPostfach(postfachId, topicHeader.getClerkMessage().id())).withRel(
					TOPIC_LINK_RELATIONSHIP_NAME));
			}
		}).toList();
	}

	Topic getTopicOfPostfach(String postfachId, String rueckfrageId) {
		var topic = TopicMapper.fromNachricht(getRueckfrageById(postfachId, rueckfrageId));

		if (!nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(topic.getTrustLevel())) {
			throw new InsufficientTrustLevelException(rueckfrageId);
		}

		return topic;
	}

	private Nachricht getRueckfrageById(String postfachId, String rueckfrageId) {
		return getRueckfragenOfPostfach(postfachId).stream().filter(nachricht -> nachricht.id().equals(rueckfrageId)).findFirst().orElseThrow();
	}

	Collection<Nachricht> getRueckfragenOfPostfach(String postfachId) {
		var events = nachrichtEventService.getNachrichtEventsOfPostfachId(postfachId);

		return events.stream().map(this::getNachrichtenOfEvent).flatMap(List::stream).toList();
	}

	private List<Nachricht> getNachrichtenOfEvent(final NachrichtEvent event) {
		return nachrichtenRemoteService.findRueckfragen(event.postfachId(), event.address());
	}

	ReplyNachricht sendRueckfrageAnswer(final ReplyNachricht nachricht) {
		nachrichtenRemoteService.sendAnswer(nachricht, nachricht.address());

		return nachricht;
	}
}
