/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.security;

import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import de.ozgcloud.antragsraum.common.NotFoundException;
import lombok.NoArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.stereotype.Service;

import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Date;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

@Service
@NoArgsConstructor
@Log4j2
public class InMemoryUserDetailService implements UserDetailsService {
	@Value("${ozgcloud.code.expire.seconds:10}")
	private long codeExpireSeconds;
	@Value("${ozgcloud.jwt.expiration.minutes}")
	private Long jwtExpirationMinutes;
	private final ConcurrentHashMap<String, User> usersMap = new ConcurrentHashMap<>();

	Cache<String, String> tmpTokenUserIdCache;

	private Cache<String, String> getTempIdCache() {
		if (Objects.isNull(tmpTokenUserIdCache)) {
			tmpTokenUserIdCache = CacheBuilder.newBuilder()
					.expireAfterAccess(codeExpireSeconds, TimeUnit.SECONDS)
					.build();
		}

		return tmpTokenUserIdCache;
	}

	public void addUser(final String tmpId, final User user) {
		var thisUser = user;
		getTempIdCache().put(tmpId, thisUser.getId());
		thisUser = user.toBuilder().tokenExpiresAt(createExpirationDateTime()).build();

		usersMap.put(thisUser.getId(), thisUser);
	}

	void setUser(final String tmpId, final User user) {
		getTempIdCache().put(tmpId, user.getId());

		usersMap.put(user.getId(), user);
	}

	User getUserByCode(String tmpId) {
		String userId = getTempIdCache().asMap().get(tmpId);

		if (StringUtils.isEmpty(userId)) {
			LOG.error("No SAML user linked to code {} found.", tmpId);
			throw new NotFoundException(User.class, tmpId);
		}
		return getUser(userId);
	}

	User getUser(String id) {
		return usersMap.get(id);
	}

	String getSamlTokenOfUser(String userId) {
		return usersMap.get(userId).getSamlToken();
	}

	@Override
	public UserDetails loadUserByUsername(String username) {
		var optionalEntry = usersMap.entrySet().stream().filter(entry -> username.equals(entry.getValue().getUsername())).findFirst();

		return optionalEntry.map(Map.Entry::getValue).orElseThrow();
	}

	Optional<User> getUser(AuthCode refreshCode) {
		return usersMap.entrySet().stream()
				.filter(entry -> refreshCode.code().equals(entry.getValue().getRefreshCode()))
				.findFirst()
				.filter(entry -> entry.getValue().isCredentialsNonExpired())
				.map(Map.Entry::getValue);
	}

	public User updateRefreshCodeOf(User user) {
		var updated = user.toBuilder()
				.refreshCode(RandomStringUtils.randomAlphanumeric(64))
				.tokenExpiresAt(createExpirationDateTime()).build();
		usersMap.put(updated.getId(), updated);
		return updated;
	}

	Date createExpirationDateTime() {
		var nowInstant =ZonedDateTime.now(ZoneId.of("UTC")).plusMinutes(jwtExpirationMinutes).toInstant();
		LOG.debug("Setting expiry date time to {}", Date.from(nowInstant).toString());
		return new Date(nowInstant.toEpochMilli());
	}

	@Scheduled(fixedRate = 4, timeUnit = TimeUnit.HOURS)
	void userCleanUp() {
		var expiredEntries = usersMap.entrySet().stream().filter(entry -> !entry.getValue().isCredentialsNonExpired()).toList();
		expiredEntries.forEach(expiredEntry -> logout(expiredEntry.getValue()));
	}

	public void logout(User user) {
		usersMap.remove(user.getId());
	}
}
