/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.ObjectFactory;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
class CallScopeTest {
    public static final String TEST = "Test";
    public static final String TEST_NEW = "TestNew";
    @Spy
    @InjectMocks
    private CallScope callScope;

    @Nested
    class TestFreshCallScope {

        @Test
        void shouldStartScope() {
            callScope.startScope();

            assertThat(CallScope.scopedObjectsHolder).isNotNull();
            assertThat(CallScope.destructionCallbacksHolder).isNotNull();
        }

        @Test
        void shouldEndScope() {
            callScope.endScope();

            verify(callScope).callAllDestructionCallbacks();
        }

        @Test
        void getConversationId() {
            var id = callScope.getConversationId();

            assertThat(id).isEqualTo(Thread.currentThread().getName());
        }
    }

    @Nested
    class TestCallScope {
        @Mock
        private ObjectFactory<Object> objectFactory;

        @BeforeEach
        void setupEmptyCallScope() {
            callScope.startScope();
            CallScope.scopedObjectsHolder.get().put(TEST, objectFactory);
        }

        @Test
        void shouldGetAdded() {
            var newObjectFactory = mock(ObjectFactory.class);
            var object = new Object();
            when(newObjectFactory.getObject()).thenReturn(object);

            var recievedObject = callScope.get(TEST_NEW, newObjectFactory);

            assertThat(recievedObject).isNotNull().isEqualTo(object);
        }

        @Test
        void shouldGetNullName() {
            var newObjectFactory = mock(ObjectFactory.class);

            assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(() -> callScope.get(null, newObjectFactory));
        }

        @Test
        void shouldGetNullFactory() {
            assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(() -> callScope.get(TEST, null));
        }

        @Test
        void shouldGetExisting() {
            var recievedObject = callScope.get(TEST, mock(ObjectFactory.class));

            assertThat(recievedObject).isNotNull().isEqualTo(objectFactory);
        }

        @Test
        void remove() {
            var res = callScope.remove(TEST);

            assertThat(res).isNotNull().isEqualTo(objectFactory);
        }

        @Test
        void removeNull() {
            assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(() -> callScope.remove(null));
        }

        @Test
        void registerDestructionCallback() {
            var callBack = mock(Runnable.class);

            callScope.registerDestructionCallback(TEST, callBack);

            assertThat(CallScope.destructionCallbacksHolder.get()).containsEntry(TEST, callBack);
        }

        @Test
        void registerDestructionCallbackNullName() {
            var callBack = mock(Runnable.class);
            assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(() -> callScope.registerDestructionCallback(null, callBack));
        }

        @Test
        void registerDestructionCallbackNullCallback() {
            assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(() -> callScope.registerDestructionCallback(TEST, null));
        }

        @Test
        void resolveContextualObject() {
            var contextualObject = callScope.resolveContextualObject(TEST);

            assertThat(contextualObject).isEqualTo(objectFactory);
        }

        @Test
        void resolveContextualObjectNullName() {
            assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(() -> callScope.resolveContextualObject(null));
        }
    }
}