/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.FileTestProperties.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.*;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.vorgang.grpc.binaryFile.GrpcFindFilesResponse;
import de.ozgcloud.vorgang.grpc.file.GrpcOzgFile;
import io.grpc.stub.StreamObserver;

@ExtendWith(MockitoExtension.class)
class FileRemoteServiceTest {
	@Spy
	@InjectMocks
	private FileRemoteService remoteService;

	@Mock
	private FileGrpcClient grpcClient;

	@Nested
	class TestLoadingFile {

		@BeforeEach
		void init() {
			var file = GrpcOzgFile.newBuilder()
			  .setName(FILE_NAME)
			  .setSize(FILE_SIZE)
			  .setContentType(CONTENT_TYPE)
			  .setId(FILE_ID).build();
			when(grpcClient.findBinaryFilesMetaData(any(), any())).thenReturn(
			  GrpcFindFilesResponse.newBuilder().addFile(file).build());
		}

		@Test
		void shouldLoadFileMetadata() {
			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file).isNotNull();
		}

		@Test
		void shouldHaveFileName() {
			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file.fileName()).isEqualTo(FILE_NAME);
		}

		@Test
		void shouldHaveFileSize() {
			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file.fileSize()).isEqualTo(FILE_SIZE);
		}

		@Test
		void shouldHaveContentType() {
			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file.contentType()).isEqualTo(CONTENT_TYPE);
		}

		@Test
		void shouldHaveFileId() {
			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file.id()).isEqualTo(FILE_ID);
		}

		@Test
		void shouldHandleEmpty() {
			when(grpcClient.findBinaryFilesMetaData(any(), any())).thenReturn(
			  GrpcFindFilesResponse.newBuilder().build());

			var file = remoteService.getFile(FILE_ID, CHANNEL_ADDRESS);

			assertThat(file).isNull();
		}
	}

	@Nested
	class TestDownloadFileContent {

		private final ByteArrayOutputStream output = new ByteArrayOutputStream();

		@Test
		void shouldCallDownloadFileObserver() {
			remoteService.downloadFileContent(FILE_ID, output, CHANNEL_ADDRESS);

			verify(grpcClient).downloadFileContent(eq(FILE_ID), any(OutputStream.class), eq(CHANNEL_ADDRESS));
		}
	}

	@Nested
	class TestUploadFile {

		@Test
		void shouldCallGrpcClient() {
			var observer = mock(StreamObserver.class);

			when(grpcClient.uploadFile(any(OzgUploadFile.class), eq(CHANNEL_ADDRESS), any())).thenReturn(observer);

			remoteService.uploadFile(OzgUploadFileTestFactory.create(), CHANNEL_ADDRESS);

			verify(grpcClient).uploadFile(any(OzgUploadFile.class), eq(CHANNEL_ADDRESS), any());
		}
	}
}