/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.FileTestProperties.CHANNEL_ADDRESS;
import static de.ozgcloud.antragsraum.attachments.FileTestProperties.DATA;
import static de.ozgcloud.antragsraum.attachments.FileTestProperties.FILE_ID;
import static de.ozgcloud.antragsraum.attachments.FileTestProperties.VORGANG_ID;
import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatExceptionOfType;
import static org.assertj.core.api.Assertions.fail;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.anyString;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.antragsraum.common.VirusFoundException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Set;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.web.multipart.MultipartFile;

@ExtendWith(MockitoExtension.class)
class FileServiceTest {
    @Spy
    @InjectMocks
    private FileService fileService;

    @Mock
    private FileRemoteService remoteService;

    @Mock
    private VirusScannerClient virusScannerClient;

    @Nested
    class TestFileUpload {

        @Mock
        private MultipartFile multipartFile;

        @BeforeEach
        void init() throws IOException {
            when(remoteService.uploadFile(any(OzgUploadFile.class), anyString())).thenReturn(
                CompletableFuture.completedFuture(FILE_ID));
            when(multipartFile.getInputStream()).thenReturn(new ByteArrayInputStream(DATA));
        }

        @Test
        void shouldCallRemoteService() {
            fileService.upload(VORGANG_ID, CHANNEL_ADDRESS, multipartFile);

            verify(remoteService, times(1)).uploadFile(any(OzgUploadFile.class), anyString());
        }

        @Test
        void shouldGetFileId() {
            var res = fileService.upload(VORGANG_ID, CHANNEL_ADDRESS, multipartFile);

            try {
                assertThat(res.get(5, TimeUnit.SECONDS)).isEqualTo(FILE_ID);
            } catch (InterruptedException | ExecutionException | TimeoutException e) {
                fail("Exception happened! " + e.getMessage());
            }
        }
    }

    @Nested
    class TestExceptionWhenSendingFile {
        @Mock
        private MultipartFile multipartFile;

        @Test
        void shouldThrowTechnicalExceptionOnUpload() throws IOException {
            doThrow(IOException.class).when(multipartFile).getInputStream();

            assertThatExceptionOfType(TechnicalException.class).isThrownBy(
                    () -> fileService.upload(VORGANG_ID, CHANNEL_ADDRESS, multipartFile))
                .withMessage("TechnicalException happened! Message: null");
        }
    }

    @Nested
    class TestLoadingFile {

        @BeforeEach
        void init() {
            when(remoteService.getFile(anyString(), anyString())).thenReturn(OzgFileTestFactory.createOzgFile());
        }

        @Test
        void shouldGetFile() {
            var file = fileService.getFile(FILE_ID, CHANNEL_ADDRESS);

            assertThat(file).isNotNull();
        }
    }

    @Nested
    class TestDownloadingFileContent {

        @Test
        void shouldCallRemoteService() {
            fileService.downloadFileContent(FILE_ID, new ByteArrayOutputStream(), CHANNEL_ADDRESS);

            verify(remoteService).downloadFileContent(anyString(), any(OutputStream.class), anyString());
        }
    }

    @Nested
    class TestVirusScanning {
        @Mock
        private MultipartFile multipartFile;

        @Test
        void shouldThrowVirusFoundExceptionWhenUploadingFile() {
            when(virusScannerClient.scan(multipartFile)).thenReturn(Set.of("VirusA", "VirusB"));

            assertThatExceptionOfType(VirusFoundException.class).isThrownBy(
                () -> fileService.createUploadFile(VORGANG_ID, multipartFile));
        }
    }
}