/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.FileTestProperties.CONTENT_TYPE;
import static de.ozgcloud.antragsraum.attachments.FileTestProperties.DATA;
import static de.ozgcloud.antragsraum.attachments.FileTestProperties.FILE_NAME;
import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatExceptionOfType;
import static org.mockito.Mockito.when;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.util.unit.DataSize;

@ExtendWith(MockitoExtension.class)
class FileValidatorTest {
    private static final String INVALID_FILE_NAME = "Testfile.exe";
    public static final MockMultipartFile TEST_FILE = new MockMultipartFile("file", FILE_NAME, CONTENT_TYPE, DATA);
    public static final MockMultipartFile TEST_INVALID_FILE =
        new MockMultipartFile("file", INVALID_FILE_NAME, CONTENT_TYPE, DATA);
    public static final MockMultipartFile TEST_TOO_LARGE_FILE =
        new MockMultipartFile("file", FILE_NAME, CONTENT_TYPE, new byte[150_000_000]);
    @InjectMocks
    private FileValidator validator;

    @Mock
    private FileProperties fileProperties;

    @Nested
    class TestDefaultValues {

        @Nested
        class TestAllowedFileTypes {

            @ParameterizedTest
            @ValueSource(strings = {"pdf", "doc", "docx", "xls", "xlsx", "txt", "png", "jpg", "jpeg", "tif", "tiff"})
            void shouldBeValidLowerCase(String suffix) {
                var file = createFile(suffix);

                var result = validator.isValid(file);

                assertThat(result).isTrue();
            }

            @ParameterizedTest
            @ValueSource(strings = {"PDF"})
            void shouldBeValidUppercase(String suffix) {
                var file = createFile(suffix);

                var result = validator.isValid(file);

                assertThat(result).isTrue();

            }
        }

        @Nested
        class TestFileSize {

            @Test
            void shouldBeValidDefault() {
                var res = validator.isValid(TEST_FILE);

                assertThat(res).isTrue();
            }

            @Test
            void shouldNotBeValid() {
                var res = validator.isValid(TEST_INVALID_FILE);

                assertThat(res).isFalse();
            }
        }

        @Nested
        class TestForbiddenFileTypes {
            @ParameterizedTest
            @ValueSource(strings = {"zip", "exe"})
            void shouldNotBeValidLowercase(String suffix) {
                var file = createFile(suffix);

                var result = validator.isValid(file);

                assertThat(result).isFalse();
            }

            @ParameterizedTest
            @ValueSource(strings = {"EXE"})
            void shouldNotBeValidUpperCase(String suffix) {
                var file = createFile(suffix);

                var result = validator.isValid(file);

                assertThat(result).isFalse();
            }
        }
    }

    @Nested
    class TestTypeWithFileProperties {
        @InjectMocks
        private FileValidator validator;

        @Mock
        private FileProperties fileProperties;

        @BeforeEach
        void init() {
            when(fileProperties.getAllowedSuffixes()).thenReturn("pdf,doc");
            //when(fileProperties.getMaxFileSize()).thenReturn(DataSize.ofMegabytes(50));
        }

        @ParameterizedTest
        @ValueSource(strings = {"pdf", "doc"})
        void shouldBeValid(String suffix) {
            var file = createFile(suffix);

            var result = validator.isValid(file);

            assertThat(result).isTrue();
        }

        @ParameterizedTest
        @ValueSource(strings = {"txt", "png"})
        void shouldNotBeValid(String suffix) {
            var file = createFile(suffix);

            var result = validator.isValid(file);

            assertThat(result).isFalse();
        }
    }

    @Nested
    class TestSizeWithFileProperties {
        @BeforeEach
        void init() {
            when(fileProperties.getMaxFileSize()).thenReturn(DataSize.ofMegabytes(50));
        }

        @Test
        void shouldBeValidProperties() {
            var res = validator.isValid(TEST_FILE);

            assertThat(res).isTrue();
        }

        @Test
        void shouldNotBeValidProperties() {
            var res = validator.isValid(TEST_TOO_LARGE_FILE);

            assertThat(res).isFalse();
        }
    }

    private MockMultipartFile createFile(String suffix) {
        return new MockMultipartFile("file", "test." + suffix, CONTENT_TYPE, DATA);
    }

    @Test
    void shouldGetFileSuffix() {
        var suffix = validator.getFileNameSuffix("test.txt");

        assertThat(suffix).isEqualTo("txt");
    }

    @Test
    void shouldThrowException() {
        assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(() -> validator.getFileNameSuffix(null));
    }
}