/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.callcontext;

import de.ozgcloud.antragsraum.common.AuthenticationHelper;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.core.Authentication;
import org.springframework.security.test.context.support.WithMockUser;

import static de.ozgcloud.antragsraum.callcontext.ContextService.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
class ContextServiceTest {
    public static final String ANTRAGSRAUM = "Antragsraum";
    @Spy
    @InjectMocks
    private ContextService contextService;

    @Mock
    private AuthenticationHelper authenticationHelper;

    @Nested
    class TestCallContext {
        @Test
        void createCallContext() {
            var grpcContext = contextService.createCallContext();

            assertThat(grpcContext).isNotNull();
        }

        @Test
        void getClientName() {
            var name = contextService.getClientName();

            assertThat(name).isEqualTo(ANTRAGSRAUM);
        }

        @Test
        void getRequestId() {
            var requestId = contextService.getRequestId();

            assertThat(requestId).isNotNull();
        }

        @Test
        void getEmptyRequestId() {
            contextService.requestAttributes = null;

            var requestId = contextService.getRequestId();

            assertThat(requestId).isEqualTo("- no request scope -");
        }

        @Nested
        class WithAuthentication {
            @Test
            void getOtherAuthenticationToken() {
                setupToken();

                var token = contextService.getSamlResponse();

                assertThat(token).isEqualTo(NO_SAML_TOKEN);
            }
        }
    }

    @Nested
    class TestCallContextMetadata {

        @BeforeEach
        void init() {
            setupToken();
        }

        @Test
        void buildCallContextMetadata() {
            var metaData = contextService.buildCallContextMetadata();

            assertThat(metaData).isNotNull();
        }

        @Test
        @WithMockUser
        void shouldHaveClientName() {
            var metaData = contextService.buildCallContextMetadata();

            assertThat(metaData.get(ContextService.createKeyOf(KEY_CLIENT_NAME)))
                    .isEqualTo(CLIENT_NAME.getBytes());
        }

        @Test
        void shouldHaveRequestId() {
            var metaData = contextService.buildCallContextMetadata();

            assertThat(metaData.get(ContextService.createKeyOf(KEY_REQUEST_ID)))
                    .isNotNull();
        }

        @Test
        void shouldHaveJwtToken() {
            var metaData = contextService.buildCallContextMetadata();

            assertThat(metaData.get(ContextService.createKeyOf(KEY_JWT_TOKEN)))
                    .isNotNull();
        }
    }

    private void setupToken() {
        var authentication = mock(Authentication.class);
        when(authenticationHelper.getAuthentication()).thenReturn(authentication);
    }

}