/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.antragsraum.attachments.FileRemoteService;
import de.ozgcloud.antragsraum.attachments.OzgFile;
import de.ozgcloud.antragsraum.attachments.OzgFileTestFactory;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrage;

@ExtendWith(MockitoExtension.class)
class NachrichtMapperTest {
	@Mock
	private FileRemoteService fileRemoteService;

	@InjectMocks
	private NachrichtMapper mapper;

	@Nested
	class TestFromGrpcRueckfrage {
		private final GrpcRueckfrage rueckfrage = GrpcRueckfragenTestFactory.create();
		private final GrpcRueckfrage answeredRueckfrage = GrpcRueckfragenTestFactory.createWithReply();

		@BeforeEach
		void setup() {
			when(fileRemoteService.getFile(anyString(), anyString())).thenReturn(OzgFileTestFactory.createOzgFile());
		}

		@Test
		void shouldMapNachrichtId() {
			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfragenTestFactory.createRueckfrageBuilder().setId(NachrichtTestFactory.ID).build(),
			  NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.id()).isEqualTo(NachrichtTestFactory.ID);
		}

		@Test
		void shouldMapAddress() {
			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfragenTestFactory.createRueckfrageBuilder().setId(NachrichtTestFactory.ID).build(),
			  NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.address()).isEqualTo(NachrichtTestFactory.ADDRESS);
		}

		@Test
		void shouldMapVorgangId() {
			var nachricht = mapper.fromGrpcRueckfrage(rueckfrage, NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.vorgangId()).isEqualTo(GrpcRueckfragenTestFactory.VORGANG_ID);
		}

		@Test
		void shouldMapVorgangName() {
			var nachricht = mapper.fromGrpcRueckfrage(rueckfrage, NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.topicTitle()).isEqualTo(NachrichtTestFactory.VORGANG_TITLE);
		}

		@Test
		void shouldMapNachrichtText() {
			var nachricht = mapper.fromGrpcRueckfrage(rueckfrage, NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.message()).isEqualTo(GrpcRueckfragenTestFactory.TEXT);
		}

		@Test
		void shouldMapCreationDate() {
			var nachricht = mapper.fromGrpcRueckfrage(rueckfrage, NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.date()).isEqualTo(GrpcRueckfragenTestFactory.SEND_AT_DATE);
		}

		@Test
		void shouldMapInvalidCreationDate() {
			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfragenTestFactory.createRueckfrageBuilder().setSentAt("huu").build(),
			  NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.date()).isZero();
		}

		@Test
		void shouldMapAttachments() {
			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfragenTestFactory.create(), NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.attachments()).hasSize(2);
		}

		@Test
		void shouldMapEmptyAttachments() {
			when(fileRemoteService.getFile(anyString(), anyString())).thenReturn(null);

			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfragenTestFactory.create(), NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.attachments()).isEmpty();
		}

		@Test
		void shouldMapTrustLevel() {
			var nachricht = mapper.fromGrpcRueckfrage(answeredRueckfrage, NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.trustLevel()).isEqualTo(NachrichtTrustLevel.LOW);
		}

		@Test
		void shouldNotHaveAnswersWhenUnanswered() {
			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfragenTestFactory.create(), NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.replyNachrichten()).isEmpty();
		}

		@Test
		void shouldMapAnswerNachrichtText() {
			var nachricht = mapper.fromGrpcRueckfrage(answeredRueckfrage, NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.replyNachrichten().getFirst().message()).isEqualTo(GrpcRueckfragenTestFactory.ANSWER_TEXT);
		}

		@Test
		void shouldMapAnsweredDate() {
			var nachricht = mapper.fromGrpcRueckfrage(answeredRueckfrage, NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.replyNachrichten().getFirst().date()).isEqualTo(GrpcRueckfragenTestFactory.ANSWERED_AT_DATE);
		}

		@Test
		void shouldMapAnswerAttachments() {
			var nachricht = mapper.fromGrpcRueckfrage(answeredRueckfrage, NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.replyNachrichten().getFirst().attachments()).hasSize(1);
		}
	}

	@Nested
	class TestToGrpcRueckfrageAnswer {
		@Test
		void shouldMapNachrichtText() {
			var nachricht = mapper.toGrpcRueckfrageAnswer(ReplyNachrichtTestFactory.create());

			assertThat(nachricht.getAnswerText()).isEqualTo(ReplyNachrichtTestFactory.TEXT);
		}

		@Test
		void shouldMapEmptyAttachments() {
			var nachricht = mapper.toGrpcRueckfrageAnswer(ReplyNachrichtTestFactory.create());

			assertThat(nachricht.getAttachmentFileIdList()).isEmpty();
		}

		@Test
		void shouldMapAttachments() {
			var nachricht = mapper.toGrpcRueckfrageAnswer(
			  ReplyNachrichtTestFactory.createBuilder()
				.attachments(List.of(OzgFile.builder().id(UUID.randomUUID().toString()).build())).build());

			assertThat(nachricht.getAttachmentFileIdList()).hasSize(1);
		}

		@Test
		void shouldNotMapEmptyAttachments() {
			var nachricht = mapper.fromGrpcRueckfrage(
			  GrpcRueckfragenTestFactory.createRueckfrageBuilder().clearAttachmentFileId().addAttachmentFileId("").build(),
			  NachrichtTestFactory.ADDRESS);

			assertThat(nachricht.attachments()).isEmpty();
		}
	}
}