/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static de.ozgcloud.antragsraum.nachricht.GrpcRueckfragenTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;

import org.apache.commons.io.FileUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import de.ozgcloud.antragsraum.common.SendTimeoutException;
import lombok.extern.log4j.Log4j2;

@SpringBootTest(properties = {
  "grpc.server.inProcessName=test2",
  "grpc.server.port=-1"
})
@SpringJUnitConfig(classes = { NachrichtenServiceGrpcTestConfiguration.class })
@DirtiesContext
@ExtendWith(MockitoExtension.class)
@Log4j2
class NachrichtenRemoteServiceITCase {
	public static final String ADDRESS = "in-process:test2";
	public static final String COMMAND_ID = "id";
	String samlResponse;

	@SpyBean
	NachrichtenRemoteService nachrichtenRemoteService;
	@Autowired
	private GrpcServerInterceptor grpcServerInterceptor;

	@BeforeEach
	void setupSamlResponse() {
		try {
			samlResponse = FileUtils.readFileToString(new File("src/test/resources/testfiles/SamlResponse.xml"), Charset.defaultCharset());
		} catch (IOException e) {
			LOG.error("Error reading SAML response data.", e);
		}
	}

	@Nested
	class TestGetNachrichten {
		@BeforeEach
		void setup() {
			initUserPasswordToken();
		}

		@Test
		void shouldLoadNachrichten() {
			var nachricht = nachrichtenRemoteService.findRueckfragen(NachrichtEventTestFactory.POSTFACH_ID, ADDRESS);

			assertThat(nachricht).isNotNull();
		}

		@Test
		void shouldHaveRueckfrageId() {
			var nachrichten = nachrichtenRemoteService.findRueckfragen(NachrichtEventTestFactory.POSTFACH_ID, ADDRESS);

			assertThat(nachrichten.get(0).id()).isEqualTo(RUECKFRAGE_ID);
		}
	}

	@Nested
	class TestSendNachricht {
		@Test
		void shouldSendNachricht() {
			initUserPasswordToken();

			var res = nachrichtenRemoteService.sendAnswer(ReplyNachrichtTestFactory.create(), ADDRESS);

			assertThat(res).isTrue();
		}

		@Test
		void shouldCheckCommand() throws Exception {
			SecurityContextHolder.getContext().setAuthentication(mock(Authentication.class));

			nachrichtenRemoteService.sendAnswer(ReplyNachrichtTestFactory.create(), ADDRESS);

			verify(nachrichtenRemoteService).waitUntilCommandIsFinished(anyString(), anyString());
			verify(nachrichtenRemoteService).checkCommandIsFinished(anyString(), anyString(), any(Authentication.class));
		}

		@Test
		void shouldThrowSendTimeoutException() {
			initUserPasswordToken();

			assertThatExceptionOfType(SendTimeoutException.class).isThrownBy(() ->
			  nachrichtenRemoteService.waitUntilCommandIsFinished(ADDRESS, COMMAND_ID)
			);
		}
	}

	private void initUserPasswordToken() {
		UsernamePasswordAuthenticationToken authentication = mock(UsernamePasswordAuthenticationToken.class);
		when(authentication.getCredentials()).thenReturn("jwt-token");
		SecurityContextHolder.getContext().setAuthentication(authentication);
	}
}
