/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.nachricht;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.Duration;
import java.time.temporal.ChronoUnit;
import java.util.UUID;
import java.util.concurrent.ExecutionException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.antragsraum.callcontext.ContextService;
import de.ozgcloud.antragsraum.common.AddressNotFoundException;
import de.ozgcloud.antragsraum.common.SendTimeoutException;
import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.nachrichten.antragraum.GrpcFindRueckfragenRequest;
import de.ozgcloud.nachrichten.antragraum.GrpcFindRueckfragenResponse;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageAnswer;
import de.ozgcloud.nachrichten.antragraum.GrpcSendRueckfrageAnswerRequest;
import io.grpc.StatusRuntimeException;

@ExtendWith(MockitoExtension.class)
public class NachrichtenRemoteServiceTest {
	public static final String ADDRESS = "test";
	public static final String POSTFACH_ID = UUID.randomUUID().toString();
	public static final String SAML_TOKEN = "saml-token";

	@Spy
	@InjectMocks
	private NachrichtenRemoteService nachrichtenRemoteService;

	@Mock
	private NachrichtenGrpcClient nachrichtenGrpcClient;

	@Mock
	private NachrichtMapper nachrichtMapper;

	@Mock
	private NachrichtProperties nachrichtProperties;

	@Mock
	private ContextService contextService;

	@Nested
	class TestNachrichtenAvailable {

		@BeforeEach
		void init() {
			when(contextService.getSamlResponse()).thenReturn(SAML_TOKEN);
			when(nachrichtMapper.fromGrpcRueckfrage(any(), anyString())).thenReturn(NachrichtTestFactory.createNachricht());
			when(nachrichtenGrpcClient.getFindRueckfragen(any(GrpcFindRueckfragenRequest.class),
			  anyString())).thenReturn(
			  GrpcFindRueckfragenResponse.newBuilder().addRueckfrage(GrpcRueckfragenTestFactory.create()).build());
		}

		@Test
		void shouldLoadNachrichtById() {
			var nachricht = nachrichtenRemoteService.findRueckfragen(POSTFACH_ID, ADDRESS);

			assertThat(nachricht).isNotEmpty();
		}

		@Test
		void shouldSetSamlToken() {
			ArgumentCaptor<GrpcFindRueckfragenRequest> requestArgumentCaptor =
			  ArgumentCaptor.forClass(GrpcFindRueckfragenRequest.class);

			nachrichtenRemoteService.findRueckfragen(POSTFACH_ID, ADDRESS);

			verify(nachrichtenGrpcClient).getFindRueckfragen(requestArgumentCaptor.capture(), anyString());
			assertThat(requestArgumentCaptor.getValue().getSamlToken()).isNotEmpty();
		}
	}

	@Nested
	class TestNoNachrichtenAvailable {

		@BeforeEach
		void init() {
			when(contextService.getSamlResponse()).thenReturn(SAML_TOKEN);

			doThrow(StatusRuntimeException.class).when(nachrichtenGrpcClient)
			  .getFindRueckfragen(any(GrpcFindRueckfragenRequest.class), anyString());
		}

		@Test
		void shouldHandleStatusRuntimeException() {
			var nachricht = nachrichtenRemoteService.findRueckfragen(POSTFACH_ID, ADDRESS);

			assertThat(nachricht).isEmpty();
		}
	}

	@Nested
	class TestSendAnswer {

		private final ReplyNachricht reply = ReplyNachrichtTestFactory.create();

		@BeforeEach
		void init() {
			when(contextService.getSamlResponse()).thenReturn(SAML_TOKEN);
			when(nachrichtMapper.toGrpcRueckfrageAnswer(any())).thenReturn(GrpcRueckfrageAnswer.newBuilder().build());
			when(nachrichtenGrpcClient.answerRueckfrage(any(GrpcSendRueckfrageAnswerRequest.class),
			  anyString())).thenReturn(GrpcRueckfragenTestFactory.createSendRueckfrageAnswerResponse());
		}

		@Test
		void shouldSendAnswer() {
			when(nachrichtenGrpcClient.getCommand(anyString(), anyString())).thenReturn(
			  GrpcCommandTestFactory.createBuilder().setStatus(GrpcCommandTestFactory.STATUS_FINISHED).build());
			when(nachrichtProperties.getSendPollIntervalDuration()).thenReturn(Duration.of(1L, ChronoUnit.SECONDS));
			when(nachrichtProperties.getTimeoutDuration()).thenReturn(Duration.of(2L, ChronoUnit.SECONDS));

			assertThat(nachrichtenRemoteService.sendAnswer(reply, ADDRESS)).isTrue();
		}

		@Test
		void shouldSetSamlResponse() {
			when(nachrichtenGrpcClient.getCommand(anyString(), anyString())).thenReturn(
			  GrpcCommandTestFactory.createBuilder().setStatus(GrpcCommandTestFactory.STATUS_FINISHED).build());
			when(nachrichtProperties.getSendPollIntervalDuration()).thenReturn(Duration.of(1L, ChronoUnit.SECONDS));
			when(nachrichtProperties.getTimeoutDuration()).thenReturn(Duration.of(2L, ChronoUnit.SECONDS));

			ArgumentCaptor<GrpcSendRueckfrageAnswerRequest> requestArgumentCaptor =
			  ArgumentCaptor.forClass(GrpcSendRueckfrageAnswerRequest.class);

			nachrichtenRemoteService.sendAnswer(reply, ADDRESS);

			verify(nachrichtenGrpcClient).answerRueckfrage(requestArgumentCaptor.capture(), anyString());
			assertThat(requestArgumentCaptor.getValue().getSamlToken()).isNotEmpty();
		}

		@Test
		void shouldThrowTimeoutException() {
			assertThatExceptionOfType(SendTimeoutException.class).isThrownBy(
			  () -> nachrichtenRemoteService.sendAnswer(reply, ADDRESS));
		}

		@Test
		void shouldThrowTechnicalExceptionBecauseOfInterruptedException() throws Exception {
			doThrow(InterruptedException.class).when(nachrichtenRemoteService).getSendResult(any());

			assertThatExceptionOfType(TechnicalException.class).isThrownBy(
			  () -> nachrichtenRemoteService.sendAnswer(reply, ADDRESS));
		}

		@Test
		void shouldThrowTechnicalExceptionBecauseOfExecutionException() throws Exception {
			doThrow(ExecutionException.class).when(nachrichtenRemoteService).getSendResult(any());

			assertThatExceptionOfType(TechnicalException.class).isThrownBy(
			  () -> nachrichtenRemoteService.sendAnswer(reply, ADDRESS));
		}
	}

	@Nested
	class TestNoTargetAddressAvailable {

		private String address;

		@BeforeEach
		void init() {
			when(contextService.getSamlResponse()).thenReturn(SAML_TOKEN);
			doThrow(AddressNotFoundException.class).when(nachrichtenGrpcClient)
			  .getFindRueckfragen(any(GrpcFindRueckfragenRequest.class), anyString());
			address = "static://localhost:9090";
		}

		@Test
		void shouldThrowAddressNotFoundException() {
			assertThatExceptionOfType(AddressNotFoundException.class).isThrownBy(
			  () -> nachrichtenRemoteService.findRueckfragen(POSTFACH_ID, address)
			);
		}
	}
}
