/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static de.ozgcloud.antragsraum.nachricht.NachrichtenService.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.List;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.antragsraum.attachments.OzgFile;
import de.ozgcloud.antragsraum.common.InsufficientTrustLevelException;
import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.antragsraum.events.NachrichtEventService;

@ExtendWith(MockitoExtension.class)
class NachrichtenServiceTest {
	@Spy
	@InjectMocks
	private NachrichtenService nachrichtenService;
	@Mock
	private NachrichtEventService nachrichtEventService;
	@Mock
	private NachrichtenRemoteService nachrichtenRemoteService;
	@Mock
	private NachrichtenTrustLevelService nachrichtenTrustLevelService;

	@Nested
	class TestGetTopicHeadersOfPostfach {
		@BeforeEach
		void init() {
			when(nachrichtEventService.getNachrichtEventsOfPostfachId(anyString())).thenReturn(List.of(NachrichtEventTestFactory.create()));
			when(nachrichtenRemoteService.findRueckfragen(any(), anyString())).thenReturn(List.of(NachrichtTestFactory.createNachricht()));
		}

		@Test
		void shouldReturnTopicHeader() {
			when(nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(any(NachrichtTrustLevel.class))).thenReturn(false);

			var result = nachrichtenService.getTopicHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			assertThat(result).hasSize(1);
		}

		@Test
		void shouldOnlyAddSelfLink() {
			var topicHeaderWithSelfLink = NachrichtTestFactory.createTopicHeader();
			topicHeaderWithSelfLink.add(linkTo(NachrichtenController.class).withSelfRel());

			when(nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(any(NachrichtTrustLevel.class))).thenReturn(false);

			var result = nachrichtenService.getTopicHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			assertThat(result.get(0)).isEqualTo(topicHeaderWithSelfLink);
		}

		@Test
		void shouldAddSelfAndTopicLink() {
			var fullyLinkedTopicHeader = NachrichtTestFactory.createTopicHeader();
			fullyLinkedTopicHeader.add(linkTo(NachrichtenController.class).withSelfRel());
			fullyLinkedTopicHeader.add(linkTo(methodOn(NachrichtenController.class).getTopicOfPostfach(NachrichtTestFactory.POSTFACH_ID,
			  fullyLinkedTopicHeader.getClerkMessage().id())).withRel(TOPIC_LINK_RELATIONSHIP_NAME));

			when(nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(any(NachrichtTrustLevel.class))).thenReturn(true);

			var result = nachrichtenService.getTopicHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			assertThat(result.get(0)).isEqualTo(fullyLinkedTopicHeader);
		}
	}

	@Nested
	class TestGetTopicOfPostfach {
		@BeforeEach
		void init() {
			final NachrichtEvent event = NachrichtEventTestFactory.create();

			when(nachrichtEventService.getNachrichtEventsOfPostfachId(anyString())).thenReturn(List.of(event));

			when(nachrichtenRemoteService.findRueckfragen(any(), anyString())).thenReturn(List.of(NachrichtTestFactory.createNachrichtBuilder()
			  .postfachId(event.postfachId())
			  .vorgangId(NachrichtTestFactory.VORGANG_ID)
			  .id(NachrichtTestFactory.ID)
			  .attachments(List.of(OzgFile.builder().id(NachrichtTestFactory.ATTACHMENT_ID_1).build()))
			  .replyNachrichten(List.of(ReplyNachrichtTestFactory.createBuilder().id(UUID.randomUUID().toString()).build()))
			  .build())
			);
		}

		@Test
		void shouldThrowInsufficientTrustLevelException() {
			when(nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(any(NachrichtTrustLevel.class))).thenReturn(false);

			assertThatExceptionOfType(InsufficientTrustLevelException.class).isThrownBy(
				() -> nachrichtenService.getTopicOfPostfach(NachrichtTestFactory.POSTFACH_ID, NachrichtTestFactory.ID))
			  .withMessage("Current user does not have the required trust level for the Rueckfrage with id '" + NachrichtTestFactory.ID + "'.");
		}

		@Test
		void shouldMapTopic() {
			when(nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(any(NachrichtTrustLevel.class))).thenReturn(true);

			var result = nachrichtenService.getTopicOfPostfach(NachrichtTestFactory.POSTFACH_ID, NachrichtTestFactory.ID);

			assertThat(result).isNotNull();
		}
	}

	@Nested
	class TestGetRueckfragenOfPostfachWithNachrichtenEventAvailable {
		final NachrichtEvent event = NachrichtEventTestFactory.create();

		@BeforeEach
		void init() {
			when(nachrichtEventService.getNachrichtEventsOfPostfachId(anyString())).thenReturn(List.of(event));

			when(nachrichtenRemoteService.findRueckfragen(any(), anyString())).thenReturn(
			  List.of(NachrichtTestFactory.createNachrichtBuilder()
				.postfachId(event.postfachId())
				.vorgangId(NachrichtTestFactory.VORGANG_ID)
				.id(NachrichtTestFactory.ID)
				.attachments(List.of(OzgFile.builder().id(NachrichtTestFactory.ATTACHMENT_ID_1).build()))
				.replyNachrichten(List.of(ReplyNachrichtTestFactory.createBuilder().id(UUID.randomUUID().toString()).build()))
				.build())
			);
		}

		@Test
		void shouldCallNachrichtRemoteService() {
			nachrichtenService.getRueckfragenOfPostfach(event.postfachId());

			verify(nachrichtenRemoteService, atMostOnce()).findRueckfragen(anyString(), anyString());
		}

		@Test
		void shouldHaveNachrichten() {
			var nachrichten = nachrichtenService.getRueckfragenOfPostfach(event.postfachId());

			assertThat(nachrichten).hasSize(1);
		}

		@Test
		void shouldHaveNachrichtenText() {
			var nachrichten = nachrichtenService.getRueckfragenOfPostfach(event.postfachId());

			assertThat(nachrichten.stream().findFirst()).isPresent().map(Nachricht::message).isNotNull();
		}

		@Test
		void shouldHavePostfachId() {
			var nachrichten = nachrichtenService.getRueckfragenOfPostfach(event.postfachId());

			assertThat(nachrichten.stream().findFirst()).isPresent().map(Nachricht::postfachId).hasValue(event.postfachId());
		}

		@Test
		void shouldHaveAttachment() {
			var nachrichten = nachrichtenService.getRueckfragenOfPostfach(event.postfachId());

			assertThat(nachrichten.stream().findFirst()).isPresent().map(Nachricht::attachments)
			  .hasValue(List.of(OzgFile.builder().id(NachrichtTestFactory.ATTACHMENT_ID_1).build()));
		}
	}

	@Nested
	class TestNachrichtWithNullAttachment {
		final NachrichtEvent event = NachrichtEventTestFactory.create();

		@BeforeEach
		void init() {
			when(nachrichtEventService.getNachrichtEventsOfPostfachId(anyString())).thenReturn(
			  List.of(event)
			);

			when(nachrichtenRemoteService.findRueckfragen(anyString(), anyString())).thenReturn(
			  List.of(NachrichtTestFactory.createNachrichtBuilder()
				.vorgangId(NachrichtTestFactory.VORGANG_ID)
				.postfachId(event.postfachId())
				.id(NachrichtTestFactory.ID)
				.build())
			);
		}

		@Test
		void shouldNotHaveAttachment() {
			var nachrichten = nachrichtenService.getRueckfragenOfPostfach(event.postfachId());

			assertThat(nachrichten.stream().findFirst()).isPresent().map(Nachricht::attachments).isEmpty();
		}
	}

	@Nested
	class TestGetRueckfragenOfPostfachWithNoNachrichtenEventAvailable {
		@BeforeEach
		void init() {
			when(nachrichtEventService.getNachrichtEventsOfPostfachId(anyString())).thenReturn(List.of());
		}

		@Test
		void shouldCallNachrichtEventService() {
			var nachrichten = nachrichtenService.getRueckfragenOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			assertThat(nachrichten).isEmpty();
		}

		@Test
		void shouldNotCallNachrichtRemoteService() {
			nachrichtenService.getRueckfragenOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			verify(nachrichtenRemoteService, never()).findRueckfragen(anyString(), anyString());
		}
	}

	@Nested
	class TestSendNachricht {
		ReplyNachricht msg = ReplyNachrichtTestFactory.create();

		@Test
		void shouldCallRemoteService() {
			nachrichtenService.sendRueckfrageAnswer(msg);

			verify(nachrichtenRemoteService).sendAnswer(any(ReplyNachricht.class), anyString());
		}
	}
}