/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.MockedStatic;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;

import de.ozgcloud.antragsraum.security.User;
import de.ozgcloud.antragsraum.security.UserTrustLevel;

class NachrichtenTrustLevelServiceTest {
	private NachrichtenTrustLevelService nachrichtenTrustLevelService;

	@Nested
	class TestNachrichtMatchesUserTrustLevel {
		@BeforeEach
		void init() {
			nachrichtenTrustLevelService = new NachrichtenTrustLevelService();
		}

		@Test
		void shouldDenyUsersWithoutTrustLevel() {
			try (MockedStatic<SecurityContextHolder> securityContextHolder = mockStatic(SecurityContextHolder.class)) {
				var mockedSecurityContext = getMockedSecurityContext(null);
				securityContextHolder.when(SecurityContextHolder::getContext).thenReturn(mockedSecurityContext);

				var resultLow = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.LOW);
				var resultSubstantial = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.SUBSTANTIAL);
				var resultHigh = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.HIGH);

				assertThat(resultLow).isFalse();
				assertThat(resultSubstantial).isFalse();
				assertThat(resultHigh).isFalse();
			}
		}

		@Test
		void shouldAllowNachrichtenWithoutTrustLevel() {
			try (MockedStatic<SecurityContextHolder> securityContextHolder = mockStatic(SecurityContextHolder.class)) {
				var mockedSecurityContext = getMockedSecurityContext(UserTrustLevel.STORK_QAA_LEVEL_1);
				securityContextHolder.when(SecurityContextHolder::getContext).thenReturn(mockedSecurityContext);

				var result = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(null);

				assertThat(result).isTrue();
			}
		}

		@Test
		void shouldAllowLowLevelNachrichten() {
			try (MockedStatic<SecurityContextHolder> securityContextHolder = mockStatic(SecurityContextHolder.class)) {
				var mockedSecurityContext = getMockedSecurityContext(UserTrustLevel.STORK_QAA_LEVEL_1);
				securityContextHolder.when(SecurityContextHolder::getContext).thenReturn(mockedSecurityContext);

				var result = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.LOW);

				assertThat(result).isTrue();
			}
		}

		@Test
		void shouldAllowLevel4Users() {
			try (MockedStatic<SecurityContextHolder> securityContextHolder = mockStatic(SecurityContextHolder.class)) {
				var mockedSecurityContext = getMockedSecurityContext(UserTrustLevel.STORK_QAA_LEVEL_4);
				securityContextHolder.when(SecurityContextHolder::getContext).thenReturn(mockedSecurityContext);

				var resultLow = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.LOW);
				var resultSubstantial = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.SUBSTANTIAL);
				var resultHigh = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.HIGH);

				assertThat(resultLow).isTrue();
				assertThat(resultSubstantial).isTrue();
				assertThat(resultHigh).isTrue();
			}
		}

		@Test
		void shouldOnlyDenyHighLevelNachrichtenForLevel3Users() {
			try (MockedStatic<SecurityContextHolder> securityContextHolder = mockStatic(SecurityContextHolder.class)) {
				var mockedSecurityContext = getMockedSecurityContext(UserTrustLevel.STORK_QAA_LEVEL_3);
				securityContextHolder.when(SecurityContextHolder::getContext).thenReturn(mockedSecurityContext);

				var resultLow = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.LOW);
				var resultSubstantial = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.SUBSTANTIAL);
				var resultHigh = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.HIGH);

				assertThat(resultLow).isTrue();
				assertThat(resultSubstantial).isTrue();
				assertThat(resultHigh).isFalse();
			}
		}

		@Test
		void shouldOnlyAllowLowLevelNachrichtenForLevel1Users() {
			try (MockedStatic<SecurityContextHolder> securityContextHolder = mockStatic(SecurityContextHolder.class)) {
				var mockedSecurityContext = getMockedSecurityContext(UserTrustLevel.STORK_QAA_LEVEL_1);
				securityContextHolder.when(SecurityContextHolder::getContext).thenReturn(mockedSecurityContext);

				var resultLow = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.LOW);
				var resultSubstantial = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.SUBSTANTIAL);
				var resultHigh = nachrichtenTrustLevelService.nachrichtMatchesUserTrustLevel(NachrichtTrustLevel.HIGH);

				assertThat(resultLow).isTrue();
				assertThat(resultSubstantial).isFalse();
				assertThat(resultHigh).isFalse();
			}
		}

		private SecurityContext getMockedSecurityContext(UserTrustLevel userTrustLevel) {
			var authentication = mock(UsernamePasswordAuthenticationToken.class);
			when(authentication.getPrincipal()).thenReturn(User.builder().trustLevel(userTrustLevel).build());
			var context = mock(SecurityContext.class);
			when(context.getAuthentication()).thenReturn(authentication);

			return context;
		}
	}
}