/*
 * Copyright (c) 2024.   Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten
 * des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.security;

import static de.ozgcloud.antragsraum.security.SHA256withRSAAndMGF1SignatureAlgorithm.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.security.Security;

import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.opensaml.core.config.ConfigurationService;
import org.opensaml.xmlsec.algorithm.AlgorithmRegistry;

public class SecurityProviderTest {
	private static final String BOUNCY_CASTLE_PROVIDER_ID = "BC";

	private final SecurityProvider securityProvider = new SecurityProvider();

	@Nested
	class TestAfterPropertiesSet {
		@BeforeEach
		void init() {
			Security.removeProvider(BOUNCY_CASTLE_PROVIDER_ID);
		}

		@Test
		void shouldNotAddBouncyCastleProvider() {
			assertThat(Security.getProvider(BOUNCY_CASTLE_PROVIDER_ID)).isNull();
		}

		@Test
		void shouldAddBouncyCastleProvider() {
			securityProvider.afterPropertiesSet();

			assertThat(Security.getProvider(BOUNCY_CASTLE_PROVIDER_ID)).isInstanceOf(BouncyCastleProvider.class);
		}

		@Test
		void shouldHandleAlgorithmRegistryIsNull() {
			securityProvider.afterPropertiesSet();

			assertThat(ConfigurationService.get(AlgorithmRegistry.class)).isNull();
		}

		@Test
		void shouldNotRegisterMukSignatureAlgorithm() {
			try (var configService = mockStatic(ConfigurationService.class)) {
				configService.when(() -> ConfigurationService.get(AlgorithmRegistry.class)).thenReturn(new AlgorithmRegistry());

				assertThat(ConfigurationService.get(AlgorithmRegistry.class).get(RSA_SHA256_MGF1_ALGORITHM_URL)).isNull();
			}
		}

		@Test
		void shouldRegisterMukSignatureAlgorithm() {
			try (var configService = mockStatic(ConfigurationService.class)) {
				configService.when(() -> ConfigurationService.get(AlgorithmRegistry.class)).thenReturn(new AlgorithmRegistry());

				securityProvider.afterPropertiesSet();

				assertThat(ConfigurationService.get(AlgorithmRegistry.class).get(RSA_SHA256_MGF1_ALGORITHM_URL)).isInstanceOf(
				  SHA256withRSAAndMGF1SignatureAlgorithm.class);
			}
		}
	}
}
