/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.security;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.saml2.provider.service.authentication.DefaultSaml2AuthenticatedPrincipal;
import org.springframework.security.saml2.provider.service.authentication.Saml2Authentication;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
class UserMapperTest {
    static final String SAML_TOKEN = "token";
    @Mock
    private Saml2Authentication auth;

    Map<String, List<Object>> attributes;

    @BeforeEach
    void setup() {
        when(auth.getSaml2Response()).thenReturn(SAML_TOKEN);

        attributes = new HashMap<>();
        attributes.put(UserMapper.NACHNAME_URN, List.of(UserTestFactory.LAST_NAME));
        attributes.put(UserMapper.VORNAME_URN, List.of(UserTestFactory.FIRST_NAME));
        attributes.put(UserMapper.POSTKORB_HANDLE_URN, List.of(UserTestFactory.POSTKORB_HANDLE));
        attributes.put(UserMapper.VERTRAUENSNIVEAU_URN, List.of(UserTestFactory.TRUST_LEVEL.getValue()));
        attributes.put(UserMapper.BK2_URN, List.of(UserTestFactory.USER_NAME));

        var principal = new DefaultSaml2AuthenticatedPrincipal(UserTestFactory.USER_ID, attributes);
        when(auth.getPrincipal()).thenReturn(principal);
    }

    @Test
    void shouldGetUsername() {
        var user = UserMapper.map(auth);

        assertThat(user.getUsername()).isEqualTo(UserTestFactory.USER_ID);
    }

    @Test
    void shouldGetFirstName() {
        var user = UserMapper.map(auth);

        assertThat(user.getFirstName()).isEqualTo(UserTestFactory.FIRST_NAME);
    }

    @Test
    void shouldGetLastName() {
        var user = UserMapper.map(auth);

        assertThat(user.getLastName()).isEqualTo(UserTestFactory.LAST_NAME);
    }

    @Test
    void shouldGetPostkornHandle() {
        var user = UserMapper.map(auth);

        assertThat(user.getPostkorbHandle()).isEqualTo(UserTestFactory.POSTKORB_HANDLE);
    }

    @Test
    void shouldGetTrustLevel() {
        var user = UserMapper.map(auth);

        assertThat(user.getTrustLevel()).isEqualTo(UserTestFactory.TRUST_LEVEL);
    }

    @Test
    void shouldGetSaml2Token() {
        var user = UserMapper.map(auth);

        assertThat(user.getSamlToken()).isEqualTo(SAML_TOKEN);
    }

    @Test
    void shouldGetUnknownAttributesEmpty() {
        var user = UserMapper.map(auth);

        assertThat(user.getUnknownAttributes()).isEmpty();
    }

    @Test
    void shouldGetUnknownAttributes() {
        attributes.put("other", List.of("value"));
        var principal = new DefaultSaml2AuthenticatedPrincipal(UserTestFactory.USER_ID, attributes);
        when(auth.getPrincipal()).thenReturn(principal);

        var user = UserMapper.map(auth);

        assertThat(user.getUnknownAttributes()).isNotEmpty().hasSize(1);
    }

    @Test
    void shouldSetRefreshCode() {
        var user = UserMapper.map(auth);

        assertThat(user.getRefreshCode()).isNotEmpty();
    }
}